<!-- Copyright 2024 Remy Blank <remy@c-space.org> -->
<!-- SPDX-License-Identifier: MIT -->

# Python

The `{exec} python` block allows executing Python code directly in the browser.

```{exec} python
:when: load
import platform
print(f"{platform.python_implementation()} {'.'.join(platform.python_version_tuple())}"
      f" on {platform.platform()}")
```

## Module setup

Module setup code can be defined as a named `{exec} python` block, to be
referenced in the `:after:` option of other blocks.

```{exec} python
:name: python-setup
:when: never
def factorial(n):
    res = 1
    for i in range(2, n + 1):
        res *= i
    return res
```

## Terminal output

The terminal output generated by an `{exec} python` block via `sys.stdout` and
`sys.stderr` (and therefore, via `print()`) is displayed in an output block.
Output to `sys.stderr` is colored.

```{exec} python
:after: python-setup
for i in range(10):
    print(f"factorial({i}) = {factorial(i)}")

import sys
sys.stderr.write("Program terminated.\n")
```

The terminal output block can be cleared with the form-feed control character
(`\x0c`).

```{exec} python
:when: click
import asyncio

for i in range(10, 0, -1):
    print(f"\x0c{i}...")
    await asyncio.sleep(1)
print("\x0cHappy new year!")
```

## Exceptions

Exceptions that propagate out of the `{exec} python` block are displayed as a
traceback on `sys.stderr`.

```{exec} python
:after: python-setup
:editable:
def outer():
    try:
        inner()
    except Exception as e:
        raise Exception("inner() failed") from e

def inner():
    raise Exception("Something is broken")

outer()
```

## Concurrency

All `{exec} python` blocks on a page are executed in a shared, single-threaded
interpreter. Therefore, only one block can run at any given time. Nevertheless,
concurrent execution is possible through `async` coroutines. The
[`asyncio`](https://docs.python.org/3/library/asyncio.html) module provides a
lot of functionality related to `async` concurrency.

```{exec} python
:when: load
import asyncio
import time

while True:
    print(f"\x0c{time.strftime('%Y-%m-%d %H:%M:%S')}")
    await asyncio.sleep(1)
```

```{exec} python
:when: load
import asyncio

i = 0
while True:
    print(f"\x0ci={i}")
    i += 1
    await asyncio.sleep(0.2)
```
